@@ -1,6 +1,6 @@
 
 /*
- * @(#)def.h	1.16 12/16/94
+ * @(#)def.h	1.20 9/6/95
  *
  *	(c) Copyright 1993-1994 by Mark Grant. All right reserved.
  *	The author assumes no liability for damages resulting from the 
@@ -38,6 +38,13 @@ typedef int		int32;
 #define DEC_NO_KEY	(-4)
 #define DEC_BAD_FILE	(-5)
 
+#define ADD_OLD_KEY	1
+#define ADD_OK		0
+#define	ADD_NO_KEY	(-1)
+#define ADD_NO_FILE	(-2)
+#define ADD_NO_TEMP	(-3)
+#define ADD_BAD_KEY	(-4)
+
 #define ERR_NONE	0
 #define ERR_NO_KEY	(-1)
 #define ERR_NO_SECRET_KEY	(-2)
@@ -56,4 +63,9 @@ extern char	*pgp_path();
 
 #ifdef SYSV
 #define bzero(a,b)	memset((a),0,(b))
+#define bcopy (s,d,l)	memcpy((d),(s),(l))
+#ifndef linux
+#define random()	rand()
+#define srandom(s)	srand((s))
+#endif
 #endif
