@@ -1,14 +1,16 @@
 
 /*
- *	@(#)gui.c	1.49 7/11/95
+ *	@(#)gui.c	1.63 9/6/95
  *
  *	Gui.c : It is intended that this file will contain as much
  *	as possible of the user-interface code, allowing privtool to
  *	be compiled for different operating systems and UI toolkits
  *	by 'simply' replacing the lowest level of functionality.
  *
- *	(c) Copyright 1993-1994 by Mark Grant. All right reserved.
- *	The author assumes no liability for damages resulting from the 
+ *	(c) Copyright 1993-1995 by Mark Grant, and by other
+ *	authors as appropriate. All right reserved.
+ *
+ *	The authors assume no liability for damages resulting from the 
  *	use of this software, even if the damage results from defects in
  *	this software. No warranty is expressed or implied.
  *
@@ -20,6 +22,12 @@
  *	Linux compatibility changes by 
  *		- David Summers (david@actsn.fay.ar.us) 6th July 1995
  *
+ *	Various changes 
+ *		- Anders Baekgaard (baekgrd@ibm.net) 10th August 1995
+ *
+ *	Make subjects line up in message list
+ *		- Richard Huveneers (richard@hekkihek.hacom.nl) 
+ *			5th September 1995
  */
 
 #include <stdio.h>
@@ -35,6 +43,7 @@
 #include "message.h"
 #include "mailrc.h"
 #include "gui.h"
+#include "windows.h"
 
 /* Maximum number of remailers to use */
 
@@ -56,7 +65,11 @@ static	char	*mail_args[] =
 static	char	*print_args[] =
 
 {
+#ifdef linux
+	"/usr/bin/lpr",
+#else
 	"/usr/ucb/lpr",
+#endif
 	NULL
 };
 
@@ -84,16 +97,33 @@ static	char	decrypt_passphrase[]="Passphrase entry (Decryption)";
 
 char	*passphrase;
 void	(*callback_proc)();
+COMPOSE_WINDOW	*callback_win;
+
+extern	char	*our_userid;
+
+#ifdef PGPTOOLS
+static	byte	md5_pass[16];
+#endif
 
 MESSAGE	*last_message_read;
 MESSAGE	*message_to_decrypt;
 MESSAGE	*displayed_message;
 
-int	deliver_flags;
+#ifdef COMPACT
+int	layout_compact = TRUE;
+#else
+int	layout_compact = FALSE;
+#endif
 
 /* Program name */
 
-char	prog_name[] = "Privtool V0.84a BETA";
+#ifdef XSAFEMAIL
+char	prog_name[] = "XSafeMail";
+#else
+char	prog_name[] = "Privtool";
+#endif
+
+char	prog_ver [] = "V0.85 BETA";
 
 /* Set the description of the message for the message list */
 
@@ -146,16 +176,23 @@ MESSAGE	*m;
 
 	/* Create message size string */
 
-	sprintf(mess_size,"%d/%d",m->lines,m->size);
-
-	if (m->subject) {
-		sprintf(mess_desc,"%3.3s %-5d %-30.30s %20.20s  %s  %-64.64s",
+	if (layout_compact) {
+		sprintf(mess_desc,"%3.3s %4d %-20.20s %17.16s %4d %-64.64s",
 			mess_type,m->number,m->email,m->date,
-				mess_size,m->subject);
+			m->lines,m->subject ? m->subject : "");
 	}
 	else {
-		sprintf(mess_desc,"%3.3s %-5d %-30.30s %20.20s  %s ",
-			mess_type,m->number,m->email,m->date,mess_size);
+		sprintf(mess_size,"%4d/%-6d",m->lines,m->size);
+
+		if (m->subject) {
+			sprintf(mess_desc,"%3.3s %-5d %-30.30s %20.20s  %s  %-64.64s",
+				mess_type,m->number,m->email,m->date,
+					mess_size,m->subject);
+		}
+		else {
+			sprintf(mess_desc,"%3.3s %-5d %-30.30s %20.20s  %s ",
+				mess_type,m->number,m->email,m->date,mess_size);
+		}
 	}
 
 	/* Clear the old description if possible */
@@ -305,6 +342,7 @@ MESSAGE	*m;
 	/* Let user know we're doing something */
 
 	set_display_footer("Displaying Message...");
+	clear_display_footer ();
 
 	/* Display sender and date */
 
@@ -324,6 +362,8 @@ m->flags &= ~(MESS_SIGNED|MESS_ENCRYPTED);
 	if (m->flags & (MESS_SIGNED|MESS_ENCRYPTED)) {
 		if (!(m->flags & MESS_VERIFIED)) {
 
+			show_busy ();
+
 			/* Either create a new buffer or clear the old one */
 
 			if (!m->decrypted)
@@ -344,6 +384,8 @@ m->flags &= ~(MESS_SIGNED|MESS_ENCRYPTED);
 				if (!passphrase) {
 					message_to_decrypt = m;
 					callback_proc = decrypt_with_passphrase;
+					callback_win = NULL;
+					clear_busy ();
 					get_passphrase(decrypt_passphrase);
 					clear_display_footer();
 					return;
@@ -361,7 +403,13 @@ m->flags &= ~(MESS_SIGNED|MESS_ENCRYPTED);
 
 			i = decrypt_message(message_contents(m),
 				m->decrypted,
-				m->signature,passphrase,FL_ASCII);
+				m->signature,passphrase,FL_ASCII,
+#ifdef PGPTOOLS
+				md5_pass
+#else
+				NULL
+#endif
+				);
 
 			/* Destroy the passphrase if we got one */
 
@@ -373,8 +421,10 @@ m->flags &= ~(MESS_SIGNED|MESS_ENCRYPTED);
 
 			if (i == DEC_BAD_PHRASE) {
 				destroy_passphrase(TRUE);
+				clear_busy ();
 				if (bad_pass_phrase_notice(ERROR_READING)) {
 					callback_proc = decrypt_with_passphrase;
+					callback_win = NULL;
 					message_to_decrypt = m;
 					get_passphrase(decrypt_passphrase);
 				}
@@ -385,9 +435,15 @@ m->flags &= ~(MESS_SIGNED|MESS_ENCRYPTED);
 			if (i == DEC_BAD_FILE) {
 				bad_file_notice (ERROR_READING);
 				m->flags &= ~(MESS_ENCRYPTED|MESS_SIGNED);
+				clear_busy ();
 				goto display_plaintext;
 			}
 
+			if (i == SIG_BAD) {
+				if (!find_mailrc("nobeepbadsig"))
+					beep_display_window ();
+			}
+
 			/* Process the error and set flags */
 
 			set_flags_from_decryption(m,i);
@@ -434,6 +490,7 @@ m->flags &= ~(MESS_SIGNED|MESS_ENCRYPTED);
 			free_message(m);
 			display_message(newm);
 
+			clear_busy ();
 			return;
 		}
 
@@ -444,6 +501,8 @@ m->flags &= ~(MESS_SIGNED|MESS_ENCRYPTED);
 
 		if (buffer_contains_key (m->decrypted))
 			show_addkey ();
+
+		clear_busy ();
 	}
 	else {
 
@@ -553,6 +612,11 @@ void	quit_proc()
 {
 	MESSAGE	*m, *om;
 
+	if (compose_windows_open()) {
+		if (dont_quit_notice_proc())
+			return;
+	}
+
 	close_mail_file ();
 	update_random();
 
@@ -598,6 +662,32 @@ void	save_and_quit_proc ()
 	quit_proc ();
 }
 
+static	void	wipe_passphrase()
+
+{
+
+	/* If we don't have a passphrase, get the value of the
+    	   passphrase item in case it was partially entered
+   	   then aborted */
+
+	if (!passphrase) {
+		passphrase = read_passphrase_string();
+	}
+
+	/* Now destroy the passphrase in the panel item */
+
+	if (passphrase) {
+		while (*passphrase) {
+			*passphrase = 0;
+			passphrase++;
+		}
+	}
+
+	/* Then set the value of the item to nothing, just in case */
+
+	clear_passphrase_string();
+}
+
 void	got_passphrase()
 
 {
@@ -608,6 +698,18 @@ void	got_passphrase()
 	passphrase = read_passphrase_string();
 	close_passphrase_window();
 
+#ifdef PGPTOOLS
+
+	/* If we have PGP Tools, we only store the MD5 */
+
+	get_md5 (passphrase, md5_pass);
+
+	/* And immediately wipe the passphrase. We keep the pointer
+	   around until the passphrase is officially cleared */
+
+	wipe_passphrase ();
+#endif
+
 	if (callback_proc) {
 		call_proc = callback_proc;
 		callback_proc = NULL;
@@ -615,7 +717,7 @@ void	got_passphrase()
 		/* Have to do it this way as callback_proc may reset
 		   the value in callback_proc on error */
 
-		(*call_proc)();
+		(*call_proc)(callback_win);
 	}
 }
 
@@ -627,30 +729,17 @@ void	destroy_passphrase(force)
 int	force;
 
 {
-	int	i;
-
 	if (security_level() > 2 || force) {
 
-		/* If we don't have a passphrase, get the value of the
-	    	   passphrase item in case it was partially entered
-	   	   then aborted */
+		/* First wipe the passphrase string */
 
-		if (!passphrase) {
-			passphrase = read_passphrase_string();
-		}
+		wipe_passphrase();
 
-		/* Now destroy the passphrase in the panel item */
-
-		if (passphrase) {
-			while (*passphrase) {
-				*passphrase = 0;
-				passphrase++;
-			}
-		}
-
-		/* Then set the value of the item to nothing, just in case */
-
-		clear_passphrase_string();
+#ifdef PGPTOOLS
+		/* Now clear the MD5 */
+		
+		bzero (md5_pass, sizeof (md5_pass));
+#endif
 		
 		/* Delete the pointer */
 
@@ -668,6 +757,8 @@ void	abort_passphrase_proc()
 	update_random();
 
 	callback_proc = NULL;
+	callback_win = NULL;
+
 	destroy_passphrase(FALSE);
 	close_passphrase_window();
 }
@@ -740,17 +831,19 @@ int	security_level()
 }
 
 
-void	setup_display(level,phrase)
+void	setup_display(level,phrase,argc,argv)
 
 int	level;
 char	*phrase;
+int	argc;
+char	**argv;
 
 {
 	security = level;
 	if (security < 2)
 		passphrase = phrase;
 
-	setup_ui(level);
+	setup_ui(level,argc,argv);
 }
 
 void	decrypt_with_passphrase()
@@ -805,7 +898,7 @@ int	i;
 static	char	*replying_message_id;
 static	char	*replying_message_sender;
 
-void	setup_send_window()
+COMPOSE_WINDOW	*setup_send_window()
 
 {
 	if (replying_message_id) {
@@ -818,7 +911,7 @@ void	setup_send_window()
 		replying_message_sender = NULL;
 	}
 
-	x_setup_send_window ();
+	return x_setup_send_window ();
 }
 /* Set up reply variables from the message we're replying to */
 
@@ -1060,12 +1153,15 @@ char	*temp;
 }
 #endif
 
-void	deliver_proc()
+void	deliver_proc(w)
+
+COMPOSE_WINDOW	*w;
 
 {
 	char	recipient[512];
 	char	subject[256];
 	char	cc[256];
+	char	bcc[256];
 	char	buff[256];
 	BUFFER	*mail_message;
 	BUFFER	*raw_message = NULL;
@@ -1074,60 +1170,81 @@ void	deliver_proc()
 	char	**userid = NULL;
 	char	**addrs = NULL;
 	char	**cc_addrs = NULL;
+	char	**bcc_addrs = NULL;
 	int	id_count = 0;
 	int	cc_count = 0;
+	int	bcc_count = 0;
 	char	*uid,*alias;
 	char	*key_name;
 	int	encrypt_flags;
 	int	i, j;
 	BUFFER	*full_list;
 	BUFFER	*cc_list;
+	BUFFER	*bcc_list;
 	char	*nym;
+	int	deliver_flags;
 
 	update_random();
+	show_busy ();
 
-	copy_to_nl(read_recipient(),recipient);
-	copy_to_nl(read_subject(),subject);
-	copy_to_nl(read_cc(),cc);
+	copy_to_nl(read_recipient(w),recipient);
+	copy_to_nl(read_subject(w),subject);
+	copy_to_nl(read_cc(w),cc);
+	copy_to_nl(read_bcc(w),bcc);
+
+	deliver_flags = read_deliver_flags (w);
 
 	full_list = new_buffer ();
 	cc_list = new_buffer ();
+	bcc_list = new_buffer ();
 
 	expand_send (full_list, recipient);
 	if (*cc)
 		expand_send (cc_list, cc);
+	if (*bcc)
+		expand_send (bcc_list, bcc);
 
 	if (!full_list->length) {
 		free_buffer (full_list);
 		free_buffer (cc_list);
+		free_buffer (bcc_list);
+		clear_busy ();
 		return;
 	}
 
 	addrs = split_list (full_list, &id_count);
 	if (*cc)
 		cc_addrs = split_list (cc_list, &cc_count);
+	if (*bcc)
+		bcc_addrs = split_list (bcc_list, &bcc_count);
 
 	if (!id_count) {
 		free_buffer (full_list);
 		free_buffer (cc_list);
+		free_buffer (bcc_list);
 
 		if (addrs)
 			free (addrs);
 		if (cc_addrs)
 			free (cc_addrs);
+		if (bcc_addrs)
+			free (bcc_addrs);
 
+		clear_busy ();
 		return;
 	}
 
 	id_count = remove_duplicates (addrs) + 2;
 	if (cc_addrs)
 		cc_count = remove_duplicates (cc_addrs);
+	if (bcc_addrs)
+		bcc_count = remove_duplicates (bcc_addrs);
 
 	if (*recipient) {
 		BUFFER	*to_send;
 
 		to_send = new_buffer();
-		read_message_to_deliver(to_send);
+		read_message_to_deliver(w, to_send);
 
 try_again:
 		/* Check for alias */
@@ -1149,9 +1266,14 @@ try_again:
 						free (addrs);
 					if (cc_addrs)
 						free (cc_addrs);
+					if (bcc_addrs)
+						free (bcc_addrs);
 					free_buffer (full_list);
 					free_buffer (cc_list);
+					free_buffer (bcc_list);
 					callback_proc = deliver_proc;
+					callback_win = w;
+					clear_busy ();
 					get_passphrase(sign_passphrase);
 					return;
 				}
@@ -1159,28 +1281,34 @@ try_again:
 
 			/* Allow for .mailrc conversion of key ids */
 
-			userid = (char **)malloc ((id_count+cc_count) * 
-				sizeof(char *));
+			userid = (char **)malloc ((id_count+cc_count +
+				bcc_count) * sizeof(char *));
 
 			i = 0;
 			for (j = 0; j < id_count && addrs [j]; j++) {
 				alias = find_pgpkey (addrs [j]);
-				if (alias) {
+				if (alias) 
 					userid [i++] = alias;
-				}
 				else
 					userid [i++] = addrs [j];
 			}
 
 			for (j = 0; j < cc_count && cc_addrs [j]; j++) {
 				alias = find_pgpkey (cc_addrs [j]);
-				if (alias) {
+				if (alias) 
 					userid [i++] = alias;
-				}
 				else
 					userid [i++] = cc_addrs [j];
 			}
 
+			for (j = 0; j < bcc_count && bcc_addrs [j]; j++) {
+				alias = find_pgpkey (bcc_addrs [j]);
+				if (alias) 
+					userid [i++] = alias;
+				else
+					userid [i++] = bcc_addrs [j];
+			}
+
 			userid [i] = NULL;
 
 			/* Get our key name for the routine */
@@ -1207,7 +1335,13 @@ try_again:
 			encrypted = new_buffer();
 			ret_val = encrypt_message(userid,to_send,
 				encrypted,encrypt_flags,passphrase,
-				key_name); 
+				key_name,
+#ifdef PGPTOOLS
+				md5_pass
+#else
+				NULL
+#endif
+				); 
 
 			free (userid);
 
@@ -1219,18 +1353,23 @@ try_again:
 				if (no_key_notice_proc(ERROR_DELIVERY)) {
 					deliver_flags &= ~DELIVER_ENCRYPT;
 
-					update_log_item(deliver_flags);
+					update_log_item(w, deliver_flags);
 					goto try_again;
 				}
 				else {
+				bad_encrypt_exit:
 					free_buffer(encrypted);
 					free_buffer(to_send);
 					free_buffer (full_list);
 					free_buffer (cc_list);
+					free_buffer (bcc_list);
 					if (addrs)
 						free (addrs);
 					if (cc_addrs)
 						free (cc_addrs);
+					if (bcc_addrs)
+						free (bcc_addrs);
+					clear_busy ();
 					return;
 				}	
 			}
@@ -1239,26 +1378,31 @@ try_again:
 				if (no_sec_notice_proc(ERROR_DELIVERY)) {
 					deliver_flags &= ~DELIVER_SIGN;
 
-					update_log_item(deliver_flags);
+					update_log_item(w, deliver_flags);
 					goto try_again;
 				}
 				else {
-					free_buffer(encrypted);
-					free_buffer(to_send);
-					free_buffer (full_list);
-					free (addrs);
-					return;
+					goto bad_encrypt_exit;
 				}	
 			}
 
 			if (ret_val == ERR_BAD_PHRASE) {
 				free_buffer(encrypted);
 				free_buffer(to_send);
-				free_buffer (full_list);
-				free (addrs);
+				free_buffer(full_list);
+				free_buffer (cc_list);
+				free_buffer (bcc_list);
+				if (addrs)
+					free (addrs);
+				if (cc_addrs)
+					free (cc_addrs);
+				if (bcc_addrs)
+					free (bcc_addrs);
 				destroy_passphrase(TRUE);
+				clear_busy ();
 				if (bad_pass_phrase_notice(ERROR_DELIVERY)) {
 					callback_proc = deliver_proc;
+					callback_win = w;
 					get_passphrase(sign_passphrase);
 				}
 				return;
@@ -1312,13 +1456,27 @@ try_again:
 				add_list_to_header (mail_message, cc_addrs);
 			}
 
+			if (bcc_addrs) {
+				add_to_buffer (mail_message, "Bcc: ", 5);
+				add_list_to_header (mail_message, bcc_addrs);
+			}
 #if 0
-			if (strcmp(nym, cuserid(0))) {
+			if (strcmp(nym, our_userid) {
 				add_to_buffer (mail_message, "From: ", 6);
 				add_to_buffer (mail_message, nym, strlen(nym));
 			}
 #endif
 
+			for (i = 1; i < MAX_EXTRA_HEADERLINES; i++) {
+				char* line = read_extra_headerline(w,i);
+
+				if (line) {
+					add_to_buffer(mail_message, 
+						line, strlen(line));
+					free(line);
+				}
+			}
+
 			add_to_buffer(mail_message,"\n",1);
 		}
 
@@ -1365,6 +1523,7 @@ try_again:
 				else {
 
 remail_error_exit:
+					clear_busy ();
 					remail_failed_notice_proc();
 					if (raw_message)
 						free_buffer (raw_message);
@@ -1372,9 +1531,14 @@ remail_error_exit:
 					free_buffer(to_send);
 					free_buffer(mail_message);
 					free_buffer (full_list);
-					free (addrs);
+					free_buffer (cc_list);
+					free_buffer (bcc_list);
+					if (addrs)
+						free (addrs);
 					if (cc_addrs)
 						free (cc_addrs);
+					if (bcc_addrs)
+						free (bcc_addrs);
 
 #ifndef MIXMASTER_STDIN
 					unlink (temp);
@@ -1399,6 +1563,19 @@ remail_error_exit:
 				i++;
 			}
 
+			i = 0;
+			while (bcc_addrs && bcc_addrs[i]) {
+				if (setup_remail_args(bcc_addrs[i], subject))
+					run_program(remail_args[0],
+					mail_message->message,
+					mail_message->length,
+					remail_args,
+					NULL);
+				else
+					goto remail_error_exit;
+				i++;
+			}
+
 #ifndef MIXMASTER_STDIN
 			unlink (temp);
 #endif
@@ -1440,21 +1617,19 @@ remail_error_exit:
 		free_buffer(to_send);
 		free_buffer(mail_message);
 		free_buffer (full_list);
-		free (addrs);
+		free_buffer (cc_list);
+		free_buffer (bcc_list);
 
+		if (addrs)
+			free (addrs);
 		if (cc_addrs)
 			free (cc_addrs);
+		if (bcc_addrs)
+			free (bcc_addrs);
 
-		close_deliver_window();
+		close_deliver_window(w);
 	}
-}
-
-void	set_deliver_flags(value)
-
-int	value;
-
-{
-	deliver_flags = value;
+	clear_busy ();
 }
 
 void	move_message_proc(s)
@@ -1643,11 +1818,13 @@ char	*s;
 
 	s = expand_filename (s);
 
+	show_busy ();
 	set_main_footer ("Saving changes and loading new mail...");
 
 	/* Save any changes to the file */
 
 	if (save_changes () < 0) {
+		clear_busy();
 		if (failed_save_notice_proc ()) {
 			clear_main_footer ();
 			return -1;
@@ -1697,11 +1874,13 @@ char	*s;
 	}
 
 	update_message_list ();
+	set_initial_scrollbar_position ();
 
 	last_message_read = NULL;
 	displayed_message = NULL;
 	message_to_decrypt = NULL;
 
+	clear_busy ();
 	clear_main_footer();
 
 	return 0;
@@ -1712,10 +1891,13 @@ void	inbox_proc()
 {
 	update_random();
 
+	show_busy ();
 	if (reading_file (default_mail_file))
 		load_new_mail ();
 	else
 		load_file_proc (default_mail_file);
+
+	clear_busy();
 }
 
 void	save_changes_proc()
@@ -1736,8 +1918,10 @@ void	done_proc()
 {
 	update_random();
 
+	show_busy ();
 	save_changes_proc ();
 	close_all_windows ();
+	clear_busy ();
 }
 
 static	int	number_cmp (m1,m2)
@@ -1811,6 +1995,7 @@ int	(*proc)();
 
 	update_random();
 
+	show_busy ();
 	n = messages.number + deleted.number;
 
 	if (n <= 0)
@@ -1849,6 +2034,7 @@ int	(*proc)();
 	messages.end->next = NULL;
 
 	free (m_list);
+	clear_busy ();
 }
 
 void	sort_by_time ()
@@ -1967,6 +2153,7 @@ int	raw;
 
 	update_random ();
 
+	show_busy ();
 	out = new_buffer ();
 
 	m = messages.start;
@@ -2034,6 +2221,7 @@ int	raw;
 	}
 
 	free_buffer (out);
+	clear_busy ();
 }
 
 void	print_cooked_proc ()
@@ -2048,4 +2236,40 @@ void	print_raw_proc ()
 	print_message_proc (TRUE);
 }
 
+void	add_key_proc ()
+
+{
+	int	err;
+
+	update_random();
+
+	if (!displayed_message)
+		return;
+
+	set_display_footer("Adding key to keyring");
+
+	show_busy ();
+	if (displayed_message->decrypted)
+		err = add_key (displayed_message->decrypted);
+	else
+		err = add_key (message_contents(displayed_message));
+	clear_busy ();
+
+	switch (err) {
+
+		case ADD_OK:
+		set_display_footer ("Key added");
+		break;
+
+		case ADD_NO_KEY:
+		case ADD_BAD_KEY:
+		bad_key_notice_proc ();
+		clear_display_footer ();
+		break;
+
+		case ADD_OLD_KEY:
+		set_display_footer ("No new keys or signatures found");
+		break;
+	}
+}
 
