@@ -1,9 +1,11 @@
 
 /*
- *	@(#)gui.h	1.14 7/6/95
+ *	@(#)gui.h	1.19 8/10/95
  *
- *	(c) Copyright 1993-1994 by Mark Grant. All right reserved.
- *	The author assumes no liability for damages resulting from the 
+ *	(c) Copyright 1993-1995 by Mark Grant, and by other
+ *	authors as appropriate. All right reserved.
+ *
+ *	The authors assume no liability for damages resulting from the 
  *	use of this software, even if the damage results from defects in
  *	this software. No warranty is expressed or implied.
  *
@@ -12,6 +14,8 @@
  *
  *			- Mark Grant (mark@unicorn.com) 29/6/94
  *
+ *	 Added layout_compact
+ *		- Anders Baekgaard (baekgrd@ibm.net) 10th August 1995
  */
 
 /* mail_reader routines */
@@ -52,6 +56,7 @@ extern	void	sort_by_subject ();
 extern  void	sort_by_sender ();
 extern	void	sort_by_size ();
 extern	void	sort_by_status ();
+extern	void	add_key_proc ();
 
 /* GUI variable definitions */
 
@@ -63,9 +68,10 @@ extern	MESSAGE_LIST	messages;
 extern	MESSAGE_LIST	deleted;
 extern	void	(*callback_proc)();
 extern	char	*passphrase;
-extern	int	deliver_flags;
+extern	int	layout_compact;
 
 extern	char	prog_name[];
+extern	char	prog_ver[];
 extern	char	default_mail_file[];
 
 /* Host-dependant UI functions */
@@ -78,6 +84,8 @@ extern	void	clear_passphrase_string();
 extern	char	*read_file_name();
 extern	char	*read_subject ();
 extern	char	*read_cc ();
+extern	char	*read_bcc ();
+extern	char	*read_extra_headerline ();
 
 #define DELIVER_SIGN	0x01
 #define DELIVER_ENCRYPT	0x02
