@@ -1,9 +1,11 @@
 
 /*
- *	@(#)mail_reader.c	1.35 7/11/95
+ *	@(#)mail_reader.c	1.44 9/9/95
  *
- *	(c) Copyright 1993-1994 by Mark Grant. All right reserved.
- *	The author assumes no liability for damages resulting from the 
+ *	(c) Copyright 1993-1995 by Mark Grant, and by other
+ *	authors as appropriate. All right reserved.
+ *
+ *	The authors assume no liability for damages resulting from the 
  *	use of this software, even if the damage results from defects in
  *	this software. No warranty is expressed or implied.
  *
@@ -19,6 +21,12 @@
  *	Linux compatibility changes by 
  *		- David Summers (david@actsn.fay.ar.us) 6th July 1995
  *
+ *	Various changes 
+ *		- Anders Baekgaard (baekgrd@ibm.net) 10th August 1995
+ *
+ *	Fix access time when saving changes
+ *		- Richard Huveneers (richard@hekkihek.hacom.nl)
+ *			9th September 1995
  */
 
 #include <stdio.h>
@@ -50,6 +58,7 @@ static	int	read_only = FALSE;
 
 extern	char	default_mail_file [];
 extern	FILE	*mail_fp;
+extern	char	*our_userid;
 
 #ifndef linux
 #ifdef SYSV
@@ -101,7 +110,7 @@ read_loop:
 
 	do {
 		c = getc(fp);
-		if (c!= EOF && c!= '\n')
+		if (c!= '\n' && c != EOF)
 			line[i++] = c;
 
 		/* Unfortunately, some people send VERY long lines !
@@ -111,7 +120,7 @@ read_loop:
 			update_line (i);
 		}
 
-	} while (c != EOF && c!= '\n');
+	} while (c!= '\n' && c != EOF);
 
 	/* Header lines can be spread out ! */
 
@@ -229,25 +238,24 @@ static	char	pgp_signed[] = "-----BEGIN PGP SIGNED MESSAGE";
 static	char	pgp_encrypted[] = "-----BEGIN PGP MESSAGE";
 
 /*
- * Take the sender string and work out the email address. Usually there
+ * Take the string and work out the email address. Usually there
  * Are two forms, either foo@bar.com (Foobar) or Foobar <foo@bar.com>
  */
 
-static	get_email_address(m)
+static	char	*get_email (s)
 
-MESSAGE	*m;
+char	*s;
 
 {
 	char	address[256];
-	char	*a,*s,*e;
+	char	*a,*e;
 
 	a = address;
-	s = m->sender;
 
 	/* Firewall */
 
 	if (!s)
-		return;
+		return NULL;
 
 	/* Ignore spaces */
 
@@ -271,7 +279,42 @@ MESSAGE	*m;
 
 	/* Copy it into the message */
 
-	m->email = strdup(address);
+	return strdup (address);
+}
+
+/* Do so for the Reply-To line */
+
+static	get_reply_to_address (m, line)
+
+MESSAGE	*m;
+char	*line;
+
+{
+	m->reply_to = get_email (line);
+}
+
+/* And for the sender */
+
+static	get_email_address(m)
+
+MESSAGE	*m;
+
+{
+	char	*address;
+
+	address = get_email (m->sender);
+
+	if (m->to && (!address || !address[0] || 
+		strcmp(address, our_userid) == 0) && find_mailrc("showto")) {
+			m->email = malloc(strlen(m->to)+5);
+			strcpy(m->email, "To: ");
+			strcat(m->email, m->to);
+			if (address)
+				free_string (address);
+	}
+	else {
+		m->email = address;
+	}
 }
 
 static	char	*header_strdup(s)
@@ -354,6 +397,24 @@ MESSAGE	*m;
 		m->sender = header_strdup(temp);
 	}
 
+	/* Check for To: */
+
+	else if (!strncasecmp(line,"To:",3)) {
+		if (m->to)
+			free_string(m->to);
+		copy_and_join(line+4,temp);
+		m->to = header_strdup(temp);
+	}
+
+	/* Check for Reply-To: */
+
+	if (!strncasecmp(line,"Reply-To:",9)) {
+		if (m->reply_to)
+			free_string (m->reply_to);
+		copy_and_join(line+10,temp);
+		get_reply_to_address (m, temp);
+	}
+
 	/* Check for Date: */
 
 	else if (!strncasecmp(line,"Date:",5)) {
@@ -543,7 +604,7 @@ MESSAGE	*m;
 
 		if (*p == ' ') {
 
-			p++;
+			while (*++p == ' ');
 			d = temp;
 
 			while (*p && *p != '\n') {
@@ -738,7 +799,7 @@ int	kill_messages;
 #endif
 		if (*p == ' ') {
 
-			p++;
+			while (*++p == ' ');
 			d = temp;
 
 			while (*p && *p != '\n') {
@@ -795,7 +856,8 @@ int	kill_messages;
 		}
 #endif
 
-		if (kill_messages && kill_user(m->email))
+		if (kill_messages && (kill_user(m->email) || 
+			(m->subject && kill_subject (m->subject))))
 			killed = TRUE;
 
 		if (maybe_cfeed(m->email))
@@ -869,11 +931,9 @@ int	kill_messages;
 			m->size = mb->length;
 		}
 
-		if (!killed) {
-			add_to_message_list_end (&messages,m);
-		}
-		else {
-			add_to_deleted (m);
+		add_to_message_list_end (&messages, m);
+		if (killed) {
+			m->flags |= MESS_DELETED;
 		}
 
 		set_file_message (m);
@@ -903,6 +963,8 @@ int	kill_messages;
 	free_buffer (mb);
 }
 
+/* Check to see if there's any new mail in the mail file */
+
 is_new_mail ()
 
 {
@@ -1012,7 +1074,7 @@ char	*s,*cc;
 	lines = count_lines(b->message,b->length);
 
 	t = time(0);
-	fprintf(fp, "From %s %s",cuserid(NULL),ctime(&t));
+	fprintf(fp, "From %s %s",our_userid,ctime(&t));
 	fprintf(fp, "To: %s\n",to);
 	if (cc && *cc)
 		fprintf(fp, "Cc: %s\n",cc);
@@ -1086,9 +1148,13 @@ int	save_all;
 	body = message_contents (m);
 	lines = count_lines(body->message,body->length);
 
+	/* Set the content-length and number of lines */
+
 	fprintf(fp,"Content-Length: %d\n",body->length);
 	fprintf(fp,"X-Lines: %d\n",lines);
 
+	/* Set the status information */
+
 	switch (m->status) {
 
 		case MSTAT_UNREAD:
@@ -1107,12 +1173,18 @@ int	save_all;
 	if (l)
 		fprintf(fp,"Status: %s\n",l);
 
+	/* Finally, now we've done the header we can output the
+	   entire message */
+
 	if (body && body->message) 
 		fprintf(fp,"\n%s\n",body->message);
 	else
 		fprintf(fp, "\n\n");
 }
 
+/* Here we append a message to a mail file, and deal with suitable
+   errors */
+
 int	append_message_to_file(m,s,save_all)
 
 MESSAGE	*m;
@@ -1134,15 +1206,23 @@ int	save_all;
 	}
 	close (fd);
 
+	/* Ok, now let's try to open the mail file */
+
 	errno = 0;
 	fp = fopen (s,"a+t");
 
+	/* Fopen failed, so unlink the lock file and return an error
+	   code */
+
 	if (!fp) {
-		printf("Fopen failed : errno = %d\n",errno);
 		unlink (lock_file);
 		return -1;
 	}
 
+	/* We need to know where the end of the file was in case we
+	   have to truncate it later on. Note that we opened for
+	   APPEND */
+
 	l = ftell (fp);
 	append_message (m, fp, save_all);
 
@@ -1157,6 +1237,8 @@ int	save_all;
 		return -1;
 	}
 
+	/* Ok, we did it, so let's delete the lock file and return */
+
 	fclose (fp);
 	unlink (lock_file);
 
@@ -1235,6 +1317,7 @@ int	save_changes ()
 	FILE	*tmp_fp;
 	int	fd, c;
 	MESSAGE	*m;
+	struct	timeval	tv[2];
 
 	if (!mail_fp)
 		return;
@@ -1317,6 +1400,16 @@ int	save_changes ()
 #endif
 	rename (tmp_file, last_file);
 
+	/*
+	** External mail notification programs like xbiff and xfaces come into
+	** action if the modification time of the mail file is larger than OR
+	** EQUAL TO the access time.
+	*/
+
+	tv[0].tv_sec = time((time_t *) 0); tv[0].tv_usec = 0;
+	tv[1].tv_sec = tv[0].tv_sec - 1;   tv[1].tv_usec = 0;
+	(void) utimes(last_file, tv);
+
 	close_mail_file ();
 
 	/* Remove the lock file */
