@@ -1,10 +1,12 @@
 
 
 /*
- * @(#)mailrc.h	1.6 7/6/95
+ * @(#)mailrc.h	1.7 8/10/95
  *
- *	(c) Copyright 1993-1994 by Mark Grant. All right reserved.
- *	The author assumes no liability for damages resulting from the 
+ *	(c) Copyright 1993-1995 by Mark Grant, and by other
+ *	authors as appropriate. All right reserved.
+ *
+ *	The authors assume no liability for damages resulting from the 
  *	use of this software, even if the damage results from defects in
  *	this software. No warranty is expressed or implied.
  *
@@ -13,6 +15,8 @@
  *
  *			- Mark Grant (mark@unicorn.com) 29/6/94
  *
+ *	 Added MAX_EXTRA_HEADERLINES
+ *		- Anders Baekgaard (baekgrd@ibm.net) 10th August 1995
  */
 
 typedef struct _entry {
@@ -30,6 +34,8 @@ typedef struct _entry {
 #define	MAILRC_PREFIXED	0x0001
 #define MAILRC_OURPREF	0x0002
 
+#define MAX_EXTRA_HEADERLINES 10
+
 extern	MAILRC	*new_mailrc();
 extern	void	free_mailrc();
 extern	char	*find_pgpkey();
