@@ -1,9 +1,11 @@
 
 /*
- *	@(#)messages.c	1.18 12/16/94
+ *	@(#)messages.c	1.20 8/10/95
  *
- *	(c) Copyright 1993-1994 by Mark Grant. All right reserved.
- *	The author assumes no liability for damages resulting from the 
+ *	(c) Copyright 1993-1995 by Mark Grant, and by other
+ *	authors as appropriate. All right reserved.
+ *
+ *	The authors assume no liability for damages resulting from the 
  *	use of this software, even if the damage results from defects in
  *	this software. No warranty is expressed or implied.
  *
@@ -12,6 +14,8 @@
  *
  *			- Mark Grant (mark@unicorn.com) 29/6/94
  *
+ *	 Added to
+ *		- Anders Baekgaard (baekgrd@ibm.net) 10th August 1995
  */
 
 #include <stdio.h>
@@ -47,6 +51,8 @@ MESSAGE	*new_message()
 	m->signature = 0;
 	m->decrypted = 0;
 	m->sender = 0;
+	m->to = 0;
+	m->reply_to = 0;
 	m->email = 0;
 	m->subject = 0;
 	m->date = 0;
@@ -84,6 +90,10 @@ MESSAGE	*b;
 
 	if (b->sender)
 		free_string (b->sender);
+	if (b->to)
+		free_string (b->to);
+	if (b->reply_to)
+		free_string (b->reply_to);
 	if (b->email)
 		free_string (b->email);
 	if (b->date)
