@@ -1,6 +1,6 @@
 
 /*
- *	@(#)pgplib.c	1.37 7/6/95
+ *	@(#)pgplib.c	1.44 8/9/95
  *
  *	pgplib.c, (mostly) by Mark Grant 1993
  *
@@ -52,6 +52,11 @@
 #endif
 
 #include "buffers.h"
+#ifdef USE_FLOPPY
+#include "floppy.h"
+#endif
+
+extern	char	*getenv();
 
 /* Following are for grabbing messages from PGP or other programs */
 
@@ -62,34 +67,7 @@ static	BUFFER	ret_messages;
 /* Following generally useful */
 
 static	char	begin_key [] = "\n-----BEGIN PGP PUBLIC KEY BLOCK-----\nVersion: ";
-
-/* All this is only needed if we're using PGPTools and not forking off
-   a copy of PGP for encryption/decryption */
-
-#ifdef PGPTOOLS
-
-/* We'll set up all the strings we need here */
-
-static	char	pgp_version [] = PGPVERSION;
-static	char	two_n [] = "\n\n";
-static	char	begin_armour[]="-----BEGIN PGP MESSAGE-----\nVersion: ";
-static	char	end_armour[]="-----END PGP MESSAGE-----\n";
-static	char	begin_signed[]="-----BEGIN PGP SIGNED MESSAGE-----\n\n";
-static	char	begin_signature[]="\n-----BEGIN PGP SIGNATURE-----\nVersion: ";
-static	char	end_signature[]="-----END PGP SIGNATURE-----\n";
-
-static	char	good_sig [] = "Good Signature from user ";
-static	char	bad_sig [] = "WARNING: Bad signature, doesn't match file contents !\nBad signature from user ";
-static	char	no_key [] = "WARNING: Can't check signature integrity, can't find public key\n         from user 0x";
-
-/* Random number generator variables */
-
-#define RAND_SIZE	256
-
-static	int	rand_pointer = 0;
-static	byte	init_rand = FALSE;
-static	byte	privseed[RAND_SIZE];
-static	byte	random_seed[RAND_SIZE];
+static	char	end_key [] = "\n-----END PGP PUBLIC KEY BLOCK-----";
 
 #ifdef CRAP_STRSTR
 
@@ -128,6 +106,34 @@ char	*s2;
 }
 #endif
 
+/* All this is only needed if we're using PGPTools and not forking off
+   a copy of PGP for encryption/decryption */
+
+#ifdef PGPTOOLS
+
+/* We'll set up all the strings we need here */
+
+static	char	pgp_version [] = PGPVERSION;
+static	char	two_n [] = "\n\n";
+static	char	begin_armour[]="-----BEGIN PGP MESSAGE-----\nVersion: ";
+static	char	end_armour[]="-----END PGP MESSAGE-----\n";
+static	char	begin_signed[]="-----BEGIN PGP SIGNED MESSAGE-----\n\n";
+static	char	begin_signature[]="\n-----BEGIN PGP SIGNATURE-----\nVersion: ";
+static	char	end_signature[]="-----END PGP SIGNATURE-----\n";
+
+static	char	good_sig [] = "Good Signature from user ";
+static	char	bad_sig [] = "WARNING: Bad signature, doesn't match file contents !\nBad signature from user ";
+static	char	no_key [] = "WARNING: Can't check signature integrity, can't find public key\n         from user 0x";
+
+/* Random number generator variables */
+
+#define RAND_SIZE	256
+
+static	int	rand_pointer = 0;
+static	byte	init_rand = FALSE;
+static	byte	privseed[RAND_SIZE];
+static	byte	random_seed[RAND_SIZE];
+
 /* Routine to open a file in $PGPPATH */
 
 static	FILE	*open_pgp_file (s,attr)
@@ -136,7 +142,7 @@ char	*s,*attr;
 
 {
 	char	temp[1024];
-	char	*pgppath, *getenv();
+	char	*pgppath;
 	FILE	*fp;
 
 	/* Try PGPPATH first */
@@ -463,6 +469,24 @@ byte	*trust;
 
 	return TRUE;
 }
+
+/* get_md5() - we use this to get the MD5 of the passphrase */
+
+void	get_md5 (pass, md5_val)
+
+char	*pass;
+byte	*md5_val;
+
+{
+	MD5_CTX	context;
+
+	MD5Init (&context);
+	MD5Update (&context, (byte *)pass, (unsigned)strlen (pass));
+	MD5Final (md5_val, &context);
+
+	bzero (&context, sizeof (context));
+}
+
 #endif
 
 void	init_pgplib()
@@ -554,6 +578,12 @@ void	close_pgplib()
 		fclose (fp);
 	}
 #endif
+
+	/* Eject floppy disk on exit */
+
+#ifdef USE_FLOPPY
+	eject_floppy();
+#endif
 }
 
 /* Clear all the output-grabbing buffers */
@@ -838,13 +868,14 @@ static	char	hex_val (i)
 }
 #endif
 
-int	decrypt_message (message,decrypted,signature,pass,flags)
+int	decrypt_message (message,decrypted,signature,pass,flags,md5_pass)
 
 BUFFER	*message;
 BUFFER	*decrypted;
 BUFFER	*signature;
 char	*pass;
 int	flags;
+byte	*md5_pass;
 
 {
 	int	ret_val, i;
@@ -1054,7 +1085,7 @@ int	flags;
 			/* Ok, it's armored, process it */
 
 			fifo_find ((byte *)"Version", temp);
-			fifo_find ((byte *)"\n", temp);
+			fifo_find ((byte *)"\n\n", temp);
 
 			/* Extract the armored file */
 
@@ -1082,16 +1113,25 @@ int	flags;
 				/* IDEA packet */
 
 				if (!got_a_key) {
-					if (!pass) {
+					if (!pass && !md5_pass) {
 						fifo_destroy (inf);
 						if (cant_decrypt)
 							free_buffer (cant_decrypt);
 						return DEC_BAD_PHRASE;
 					}
 
-					MD5Init (&context);
-					MD5Update (&context, (byte *)pass, (unsigned)strlen (pass));
-					MD5Final (idea_key, &context);
+					if (!md5_pass) {
+						MD5Init (&context);
+						MD5Update (&context, 
+							(byte *)pass, 
+							(unsigned)strlen (pass));
+						MD5Final (idea_key, &context);
+					}
+					else {
+						bcopy (md5_pass,
+							idea_key,
+							sizeof(idea_key));
+					}
 
 					bzero (&context, sizeof (context));
 				}
@@ -1143,7 +1183,12 @@ int	flags;
 				mess = fifo_copy (inf);
 
 				pgp_get_keyid (inf, key);
+#ifdef USE_FLOPPY
+				skr = get_flop_file ();
+				rewind (skr);
+#else
 				skr = open_pgp_file ("secring.pgp", "rb");
+#endif
 
 				if (!skr) {
 				failed_decrypt_no_key:
@@ -1163,7 +1208,9 @@ int	flags;
 					int	c;
 
 					fifo_destroy (keyring);
+#ifndef USE_FLOPPY
 					fclose (skr);
+#endif
 
 					if (!cant_decrypt) {
 						cant_decrypt = new_buffer ();
@@ -1215,14 +1262,16 @@ int	flags;
 				if (pgpk_extract_key (temp, &public,
 					&secret, NULL, NULL)) {
 
-					if (!pass) {
+					if (!pass && !md5_pass) {
 						bzero (&secret, sizeof (secret));
 						bzero (&public, sizeof (public));
 						bzero (key, 16);
 						fifo_destroy (temp);
 						fifo_destroy (keyring);
 						fifo_destroy (inf);
+#ifndef USE_FLOPPY
 						fclose (skr);
+#endif
 
 						if (cant_decrypt)
 							free_buffer (cant_decrypt);
@@ -1234,11 +1283,17 @@ int	flags;
 
 					/* Calculate the MD5 of the passphrase */
 
-					MD5Init (&context);
-					MD5Update (&context, (byte *)pass, (unsigned)strlen (pass));
-					MD5Final (phrase_md5, &context);
+					if (!md5_pass) {
+						MD5Init (&context);
+						MD5Update (&context, (byte *)pass, (unsigned)strlen (pass));
+						MD5Final (phrase_md5, &context);
+					}
+					else
+						bcopy (md5_pass,
+							phrase_md5,
+							sizeof (phrase_md5));
 
-					bzero (context, sizeof (context));
+					bzero (&context, sizeof (context));
 					fifo_destroy (temp);
 
 					/* Decrypt the secret key */
@@ -1248,7 +1303,9 @@ int	flags;
 						/* Probably isn't neccesary, but just for luck ! */
 
 						fifo_destroy (keyring);
+#ifndef USE_FLOPPY
 						fclose (skr);
+#endif
 
 						bzero (phrase_md5, 16);
 						bzero (&secret, sizeof (secret));
@@ -1265,7 +1322,9 @@ int	flags;
 				}
 
 				fifo_destroy (keyring);
+#ifndef USE_FLOPPY
 				fclose (skr);
+#endif
 
 				if (!pgp_extract_rsa (inf, idea_key, &public,
 					&secret)) {
@@ -1362,7 +1421,7 @@ int	flags;
 					MD5Update (&context, (byte *) &etimestamp, 4);
 					MD5Final (mess_md5, &context);
 
-					bzero (context, sizeof (context));
+					bzero (&context, sizeof (context));
 
 					/* Verify signature and create return info */
 
@@ -1498,7 +1557,7 @@ int	flags;
 
 /* Note - user is a list of users, not just one ! */
 
-int	encrypt_message(user,message,encrypted,flags,pass,key_name)
+int	encrypt_message(user,message,encrypted,flags,pass,key_name,md5_pass)
 
 char	**user;
 BUFFER	*message;
@@ -1506,6 +1565,7 @@ BUFFER	*encrypted;
 int	flags;
 char	*pass;
 char	*key_name;
+byte	*md5_pass;
 
 {
 #ifdef PGPTOOLS
@@ -1547,7 +1607,7 @@ char	*key_name;
 
 		/* Oops, no passphrase */
 
-		if (!pass) {
+		if (!pass && !md5_pass) {
 			if (pkr)
 				fclose (pkr);
 			if (smkr)
@@ -1558,7 +1618,12 @@ char	*key_name;
 
 		/* Open the secret key file */
 
+#ifdef USE_FLOPPY
+		skr = get_flop_file ();
+		rewind (skr);
+#else
 		skr = open_pgp_file ("secring.pgp", "rb");
+#endif
 
 		/* Check the file is there ! */
 
@@ -1579,8 +1644,10 @@ char	*key_name;
 
 		/* Close the keyring */
 
+#ifndef USE_FLOPPY
 		if (skr)
 			fclose (skr);
+#endif
 
 		/* Did we get it ? */
 
@@ -1602,11 +1669,17 @@ char	*key_name;
 
 			/* Calculate the MD5 of the passphrase */
 
-			MD5Init (&context);
-			MD5Update (&context, (byte *)pass, (unsigned)strlen (pass));
-			MD5Final (phrase_md5, &context);
+			if (!md5_pass) {
+				MD5Init (&context);
+				MD5Update (&context, (byte *)pass, (unsigned)strlen (pass));
+				MD5Final (phrase_md5, &context);
+			}
+			else 
+				bcopy (md5_pass,
+					phrase_md5,
+					sizeof (phrase_md5));
 
-			bzero (context, sizeof (context));
+			bzero (&context, sizeof (context));
 
 			/* Decrypt the secret key */
 
@@ -2107,3 +2180,185 @@ BUFFER	*b;
 	return FALSE;
 }
 
+/* Add a PGP key to the pubring.pgp file */
+
+int	add_key (m)
+
+BUFFER	*m;
+
+{
+	char	*s, *e;
+	char	*dir;
+#ifdef PGPTOOLS
+	struct	fifo	*armor;
+	struct	fifo	*new_key;
+	struct	fifo	*n_kr;
+	struct	fifo	*o_kr;
+	struct	fifo	*temp;
+	struct	pgp_pubkey	pk, pkt;
+	byte	trust;
+	char	in_file[1024];
+	char	out_file[1024];
+	char	back_file[1024];
+	FILE	*opr;
+	FILE	*npr;
+#else
+	FILE	*temp_fp;
+	char	temp_name [1024];
+	char	*argv[5];
+#endif
+
+	if (!m->message || !m->length)
+		return ADD_NO_KEY;
+
+	if (!strncmp ((char *)m->message, begin_key + 1, 
+		strlen (begin_key) - 1)) {
+		s = (char *)m->message;
+	}
+	else
+		s = strstr ((char *)m->message, begin_key);
+
+	if (!s || s >= (char *)(m->message + m->length))
+		return ADD_NO_KEY;
+
+	e = strstr ((char *)m->message, end_key);
+
+	if (!e || e >= (char *)(m->message + m->length))
+		return ADD_NO_KEY;
+
+#ifdef PGPTOOLS
+	s += strlen (begin_key);
+
+	while (*s && *s != '\n')
+		s++;
+
+	if (*s)
+		s++;
+	else
+		return ADD_NO_KEY;
+
+	/* We now have s pointing at the beginning of the armor */
+
+	armor = fifo_mem_create ();
+	fifo_aput (s, (e - s) + 1, armor);
+
+	new_key = fifo_mem_create ();
+	if (!pgp_extract_armor (armor, new_key)) {
+		fifo_destroy (armor);
+		fifo_destroy (new_key);
+		return ADD_BAD_KEY;
+	}
+	fifo_destroy(armor);
+
+	dir = getenv ("PGPPATH");
+	if (!dir) {
+	addkey_no_file:
+		fifo_destroy (new_key);
+		return ADD_NO_FILE;
+	}
+
+	sprintf(in_file, "%s/pubring.pgp", dir);
+	sprintf(out_file, "%s/newring.pgp", dir);
+	sprintf(back_file, "%s/pubring.bak", dir);
+
+	opr = fopen (in_file, "rb");
+	npr = fopen (out_file, "wb");
+	if (!npr)
+		goto addkey_no_file;
+
+	if (opr) {
+		char	userid[STRING_SIZE];
+
+		/* Check to see if the key already exists */
+
+		temp = fifo_copy (new_key);
+		pgpk_extract_key (temp, &pk, NULL, NULL, NULL);
+		fifo_destroy (temp);
+		if (get_public_key (pk.keyid, userid, &pkt, &trust)) {
+			/* Key already exists */
+
+			bzero (&pk, sizeof (pk));
+			bzero (&pkt, sizeof (pkt));
+
+			fifo_destroy (new_key);
+			fclose (npr);
+			fclose (opr);
+
+			return ADD_OLD_KEY;
+		}
+		
+		o_kr = fifo_file_create (opr);
+	}
+	else
+		o_kr = fifo_mem_create ();
+
+	n_kr = fifo_file_create (npr);
+	pgpk_keyring_add (o_kr, n_kr, new_key);
+
+	fifo_destroy (n_kr);
+	fclose (npr);
+	if (opr)
+		fclose (opr);
+
+	fifo_destroy (new_key);
+
+	/* Now copy the files over */
+
+	unlink (back_file);
+	rename (in_file, back_file);
+	rename (out_file, in_file);
+#else
+	/* OK, we now have the key, write it to a temporary file, then
+	   call PGP to add it. We would use 'pgp -f', but it doesn't
+	   like adding keys that way. */
+
+	dir = getenv ("PGPPATH");
+
+	if (!dir) {
+#ifdef MOST_SECURE
+		return ADD_NO_TEMP
+#else
+		strcpy (temp_name, "new-keys.asc");
+#endif
+	}
+	else
+		sprintf(temp_name, "%s/new-keys.asc", dir);
+
+	temp_fp = fopen(temp_name, "wt");
+
+	if (!temp_fp)
+		return ADD_NO_TEMP;
+
+	/* Write out the key */
+
+	fwrite (s, (e - s) + strlen (end_key) + 1, 1, temp_fp);
+	fclose (temp_fp);
+
+	/* So, it's now in the temporary file, call PGP */
+
+	argv[0] = "pgp";
+	argv[1] = "-ka";
+	argv[2] = "+batchmode=on";
+	argv[3] = temp_name;
+	argv[4] = NULL;
+
+	run_pgp (NULL, 0, argv, NULL);
+
+	/* Having done that, unlink the temporary file */
+
+	unlink (temp_name);
+
+	/* Check for new keys */
+
+	if (strstr((char *)stdout_messages.message,"No new keys"))
+		return ADD_OLD_KEY;
+
+	if (strstr((char *)stdout_messages.message,"Keyring add error"))
+		return ADD_BAD_KEY;
+
+	/* OK, if we get here, then it ought to have worked.. */
+#endif
+
+	return	ADD_OK;
+}
+
